#!/usr/bin/perl -w

###########################################################################
# This script takes a CNF formula, translates it into DIMACS format,
# feeds it into Z3, and interprets the result.
# A formula is a list of clauses, each in one line, with "-" for negation.
# E.g., the following file contents
#	a -b
#       c
# correspond to the formula (a \/ (not) b) /\ c.
###########################################################################

use IPC::Open2;

$PROG = "cnfcheck";

$optdisp = 0;
$fn = shift @ARGV;
if ($fn eq "-d") {
	$fn = shift @ARGV;
	$optdisp = 1;
}
if (defined($fn)) {
	if ($fn eq "-h") {
		print "\nusage: $PROG [-d] [<filename>] ",
				"-- satisfiability test for CNF\n",
			"  -d: only show DIMACS output, no solving\n",
			"  <filename> : contains formula\n",
			"  if no filename is given, reads from stdin\n\n";
		exit 2;
	}
	die "cannot read file $fn" unless -r $fn;
	open STDIN,$fn;
	print STDERR "$PROG -- (invoke with -h for help)\n";
	print STDERR "reading formula from file $fn\n";
} else {
	print STDERR "$PROG -- (invoke with -h for help)\n";
	print STDERR "reading formula from standard input\n";
}

if (!$optdisp) {
	open2(*Reader,*Writer,"z3 -dimacs -in");
	select Writer;
}

print "p cnf\n";

$c = 0;
while (<>) {
	tr/\015\012//d;
	@names = split(/\s+/);
	next if $#names < 0;
	for $m (@names) {
		next if $m eq "";
		($neg,$n) = ($m =~ /(-?)(.*)/);
		if (defined($h{$n})) { $i = $h{$n}; }
				else { $i = $h{$n} = ++$c; $v[$i] = $n; }
		print "$neg$i ";
	}
	print "0\n";
}

exit 0 if $optdisp;

select STDOUT;
close Writer;

while (<Reader>) {
	tr/\015\012//d;
	if (/^unsat$/) {
		print "The formula is UNSATISFIABLE.\n";
		exit 1;
	}
	if (/^sat$/) {
		print "The formula is SATISFIABLE as follows:\n";
		next;
	}
	@a = split;
	for $l (@a) {
		$i = ($l > 0)? $l : -$l;
		print $v[$i]," = ",($l > 0)? "true" : "false","\n";
	}
}

exit 0;
